///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAdmScreen.cc  
// ----------------
// Cego admin screen
//
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-2022 Bjoern Lemke
//
// IMPLEMENTATION MODULE
//
// Class: CegoAdmScreen
// 
// Description: Cursor based administration database administration frontend
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <lfcbase/Tokenizer.h>

#include "CegoAdmScreen.h"
#include "CegoXMLdef.h"
#include "CegoDefs.h"

#define INPUT_TYPE_STRING "S"
#define INPUT_TYPE_ID "I"
#define INPUT_TYPE_NUMBER "N"
#define INPUT_TYPE_PWD "P"
#define INPUT_TYPE_MENU "M"

#define MAXNAMELEN "20"
#define MAXSIZELEN "20"
#define MAXPATHLEN "80"
#define MAXPASSWDLEN "20"

#define DEFREFINT 3000

// action defines

#define TS_EDIT 1
#define TS_CREATE_ACTION 2
#define TS_START_ACTION 3
#define TS_STOP_ACTION 4
#define TS_DROP_ACTION 5
#define TS_REMOVE_ACTION 6
#define TS_INFO_ACTION 7
#define TS_FILE 8
#define TS_ARCHLOG 9
#define TS_BACKUP 10
#define TS_QUERYCACHE 11
#define TS_EXPORT 12
#define TS_IMPORT 13


// ts status definition
#define TS_DEFINED_STATUS "DEFINED"
#define TS_ONLINE_STATUS "ONLINE"
#define TS_OFFLINE_STATUS "OFFLINE"

// user action defines
#define USER_TOGGLETRACE_ACTION 1
#define USER_DELETE_ACTION 2
#define USER_ASSIGNROLE_ACTION 3
#define USER_REMOVEROLE_ACTION 4
#define USER_CHANGEPWD_ACTION 5

#define ERROR_TITLE "ERROR"
#define INFO_TITLE "INFORMATION"

#define MSGBOX_WIDTH 60

#define HELPMASK_WIDTH 50
#define HELPMASK_HEIGHT 30

#define VAL_MAX_LEN 30
#define VAL_MAX_NUM 20

#define INFO_ACTION 100
#define ADD_ACTION 200
#define DELETE_ACTION 300
#define FILEINFO_ACTION 400
#define RESTORE_ACTION 500
#define CONF_ACTION 600

#define ATTRSEPTOKEN Chain("#")
#define LISTSEPTOKEN Chain("&")
#define VALSEPTOKEN Chain("=")

CegoAdmScreen::CegoAdmScreen(CegoAdminHandler *pAH) : Screen(ATTRSEPTOKEN, LISTSEPTOKEN, VALSEPTOKEN)
{
    _pAH = pAH;
    _refInterval = DEFREFINT;
}

CegoAdmScreen::~CegoAdmScreen()  
{    
}

void CegoAdmScreen::showScreen()
{
    regMenu(Chain("BufferPool"), 1); 
    regMenu(Chain("ThreadPool"), 2); 
    regMenu(Chain("TableSet"), 3);
    regMenu(Chain("Role"), 4);
    regMenu(Chain("User"), 5);
    regMenu(Chain("Manager"), 6); 
    regMenu(Chain("Refresh"), 7);
    regMenu(Chain("Quit"), 8);

    regShortCut('i', INFO_ACTION);
    regShortCut('a', ADD_ACTION);
    regShortCut('d', DELETE_ACTION);
    regShortCut('f', FILEINFO_ACTION); 
    regShortCut('r', RESTORE_ACTION);
    regShortCut('c', CONF_ACTION);
    
    int midx = 0; 
    while ( midx != 8 ) 
    { 
	setStatusLine(Chain("<cur-left>,<cur-right> - navigate to menu"));
	
	midx = showHeader(); 

	if ( midx == 1 ) 
	{ 
	    showPool();
	}
	else if ( midx == 2 )
	{
	    showThread();
	}
	else if ( midx == 3 )
	{
	    showTableSet();
	}
	else if ( midx == 4 )
	{
	    showRole();
	}
	else if ( midx == 5 )
	{
	    showUser();
	}
	else if ( midx == 6 )
	{
	    showManager();
	}
	else if ( midx == 7 )
	{
	    showRefreshMenu();
	}	
    }    
}


/////////////////////
// Bufferpool Menu //
/////////////////////

void CegoAdmScreen::showPool()
{    
    int ret = 1;
    
    while ( ret != SC_LEAVE  )
    {
	setStatusLine(Chain("<ret>,<esc> - return to menu"), _refInterval / 1000 + 1);
	
	ListT<ListT<CegoFieldValue> > poolinfo1;
	ListT<ListT<CegoFieldValue> > poolinfo2;
	
	CegoAdminHandler::ResultType res;
	CegoTableObject oe;
	
	res = _pAH->reqPoolInfo();
	
	_pAH->getPoolInfo(oe, poolinfo1, poolinfo2);	    	    

	ListT< ListT<Chain> > poolInfoAll;

	ListT<Chain> poolInfoA;
	ListT<CegoFieldValue> *pFVL = poolinfo1.First();
	while ( pFVL )
	{
	    Chain key, value;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {		       
		key = pFV->valAsChain();
	    }
	    
	    pFV = pFVL->Next();
	    if ( pFV )
	    {
		value = pFV->valAsChain();
	    }
	    
	    poolInfoA.Insert(key + VALSEPTOKEN + value);	    	    
	    pFVL = poolinfo1.Next();
	}

	poolInfoAll.Insert(poolInfoA);

	ListT<Chain> poolInfoB;
	pFVL = poolinfo2.First();
	while ( pFVL )
	{
	    Chain key, value;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {		       
		key = pFV->valAsChain();
	    }
	    
	    pFV = pFVL->Next();
	    if ( pFV )
	    {
		value = pFV->valAsChain();
	    }
	    
	    poolInfoB.Insert(key + VALSEPTOKEN + value);	    	    
	    pFVL = poolinfo2.Next();
	}

	poolInfoAll.Insert(poolInfoB);

	ret = showAttributeBox(20, 25, poolInfoAll, false, _refInterval);
    }
}

/////////////////
// Thread Menu //
/////////////////

void CegoAdmScreen::showThread()
{
    int ret = SC_TIMEOUT;

    ListT<Chain> threadSchema;
    threadSchema.Insert(Chain("Name") + VALSEPTOKEN + Chain("10"));
    
    ListT< ListT<Chain> > threadTable;	
    
    ListT<Chain> dbRow;
    dbRow.Insert("DB Pool");	
    threadTable.Insert(dbRow);
    
    ListT<Chain> admRow;
    admRow.Insert("Admin Pool");	
    threadTable.Insert(admRow);
    
    ListT<Chain> logRow;
    logRow.Insert("Log Pool");	
    threadTable.Insert(logRow);
    
    ListT<Screen::TableColor> colorMap;		   
    // colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

    setStatusLine(Chain("<ret> - select thread pool"));
    
    ret = showTableBox(Chain("Thread Pool"), threadSchema, threadTable, colorMap, 0, true, 1, 1, 3);
    if ( ret == SC_SELECT )
    {
	int roleIdx = getSelectedRow() - 1;	    
	
	Chain poolName = threadTable[roleIdx][0];
	
	if ( poolName == Chain("DB Pool") )
	{
	    showDBThread();
	}
	else if ( poolName == Chain("Admin Pool") )
	{
	    showAdmThread();
	}
	else if ( poolName == Chain("Log Pool") )
	{
	    showLogThread();
	}      
    }
}

void CegoAdmScreen::showDBThread()
{
    int ret = -1;
    while ( ret != SC_LEAVE && ret != SC_SELECT )
    {

	setStatusLine(Chain("i - thread information, <esc>,<ret> - return to menu"), _refInterval / 1000 + 1);
		
	ListT<ListT<CegoFieldValue> > dbthreadinfo;
	
	CegoAdminHandler::ResultType res;
	CegoTableObject oe;
	Chain format;    
	
	res = _pAH->reqDbThreadInfo();
	_pAH->getDbThreadInfo(oe, dbthreadinfo, format);

	ListT<Chain> threadSchema;
	threadSchema.Insert(Chain("ThreadId") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("NumConReq") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("NumQueryReq") + VALSEPTOKEN + Chain("20"));
	threadSchema.Insert(Chain("Load") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("SortAlloc") + VALSEPTOKEN + Chain("15"));
	threadSchema.Insert(Chain("Status") + VALSEPTOKEN + Chain("10"));
		
	ListT<ListT<Chain> > threadTable;
		   
	ListT<CegoFieldValue> *pFVL = dbthreadinfo.First();
	while ( pFVL )
	{
	    ListT<Chain> threadRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while ( pFV )
	    {
		threadRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    threadTable.Insert(threadRow);
	    pFVL = dbthreadinfo.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	colorMap.Insert(Screen::TableColor(Chain("BUSY"), SC_RED));
	colorMap.Insert(Screen::TableColor(Chain("READY"), SC_GREEN));
	colorMap.Insert(Screen::TableColor(Chain("CONNECTED"), SC_YELLOW));

	ret = showTableBox(Chain("DB Threads"), threadSchema, threadTable,  colorMap, _refInterval, true);
	if ( ret == INFO_ACTION )
	{
	    int selectedThread = getSelectedRow() - 1;
	    showLastThreadAction(selectedThread);
	}
    }
    setStatusLine(Chain(), _refInterval / 1000 + 1);
}

void CegoAdmScreen::showLastThreadAction(int threadId)
{
    CegoTableObject oe;
    ListT<ListT<CegoFieldValue> > info;    
    Chain format;
    
    _pAH->getDbThreadLastQuery(threadId, oe, info, format);
    
    ListT<CegoFieldValue> *pFVL = info.First();
    if ( pFVL )
    {
	CegoFieldValue *pFV = pFVL->First();
	if ( pFV )
	{
	    showInfoBox("Last action", pFV->valAsChain(), MSGBOX_WIDTH);
	}
    }
}

void CegoAdmScreen::showAdmThread()
{
    int ret = -1;
    while ( ret != SC_LEAVE
	&& ret != SC_SELECT )
    {

	setStatusLine(Chain("<esc>,<ret> - return to menu"), _refInterval / 1000 + 1);
		
	ListT<ListT<CegoFieldValue> > admthreadinfo;
	
	CegoAdminHandler::ResultType res;
	CegoTableObject oe;
	Chain format;    
	
	res = _pAH->reqAdmThreadInfo();
	_pAH->getAdmThreadInfo(oe, admthreadinfo, format);


	ListT<Chain> threadSchema;
	threadSchema.Insert(Chain("ThreadId") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("NumRequest") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("Load") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("Status") + VALSEPTOKEN + Chain("10"));
		
	ListT<ListT<Chain> > threadTable;
		   
	ListT<CegoFieldValue> *pFVL = admthreadinfo.First();
	while ( pFVL )
	{
	    ListT<Chain> threadRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while ( pFV )
	    {
		threadRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    threadTable.Insert(threadRow);
	    pFVL = admthreadinfo.Next();
	}
	
	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

	ret = showTableBox(Chain("Admin Threads"), threadSchema, threadTable, colorMap, _refInterval);

    }
}

void CegoAdmScreen::showLogThread()
{
    int ret = -1;
    while ( ret != SC_LEAVE
	&& ret != SC_SELECT )
    {
	setStatusLine(Chain("<esc>,<ret> - return to menu"), _refInterval / 1000 + 1);
	
	ListT<ListT<CegoFieldValue> > logthreadinfo;
	
	CegoAdminHandler::ResultType res;
	CegoTableObject oe;
	Chain format;    
    
	res = _pAH->reqLogThreadInfo();
	_pAH->getLogThreadInfo(oe, logthreadinfo, format);

	ListT<Chain> threadSchema;
	threadSchema.Insert(Chain("ThreadId") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("NumRequest") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("Load") + VALSEPTOKEN + Chain("10"));
	threadSchema.Insert(Chain("Status") + VALSEPTOKEN + Chain("10"));
		
	ListT<ListT<Chain> > threadTable;
	
	ListT<CegoFieldValue> *pFVL = logthreadinfo.First();
	while ( pFVL )
	{
	    ListT<Chain> threadRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while ( pFV )
	    {
		threadRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    threadTable.Insert(threadRow);
	    pFVL = logthreadinfo.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

	ret = showTableBox(Chain("Log Threads"), threadSchema, threadTable, colorMap, _refInterval);


    }
}


///////////////////
// Tableset Menu //
///////////////////

void CegoAdmScreen::showTableSet()
{    
    int ret = SC_TIMEOUT;
    int selectedRow=1;
    int visibleRow=1;
    
    while ( ret != SC_LEAVE )
    {

	setStatusLine(Chain("a - define tableset, i - show tableset info, <ret> - open action menu"));
	
	CegoAdminHandler::ResultType res;
	CegoTableObject oe;	

	ListT<ListT<CegoFieldValue> > tsList;
    
	res = _pAH->medGetTableSetList(true);

	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    res = _pAH->nextInfo();	    
	}

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	    
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	
	_pAH->getTableSetList(oe, tsList, true);

	ListT<Chain> tsSchema;
	tsSchema.Insert(Chain("Name") + VALSEPTOKEN + Chain("20"));
	tsSchema.Insert(Chain("RunState") + VALSEPTOKEN + Chain("10"));
	tsSchema.Insert(Chain("SyncState") + VALSEPTOKEN + Chain("10"));
	tsSchema.Insert(Chain("ArchMode") + VALSEPTOKEN + Chain("10"));
	tsSchema.Insert(Chain("TempUsage") + VALSEPTOKEN + Chain("10"));
	tsSchema.Insert(Chain("AppUsage") + VALSEPTOKEN + Chain("10"));
		
	ListT<ListT<Chain> > tsTable;
		   
	ListT<CegoFieldValue> *pFVL = tsList.First();
	while ( pFVL )
	{
	    ListT<Chain> tsRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while ( pFV )
	    {
		tsRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    tsTable.Insert(tsRow);
	    pFVL = tsList.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

	ret = showTableBox(Chain("Tablesets"), tsSchema, tsTable, colorMap, 0,
			   true, selectedRow, visibleRow, 0);

	selectedRow=getSelectedRow();
	visibleRow=getVisibleRow();
	
	if ( ret == SC_SELECT)
	{
	    int tsIndex = getSelectedRow() - 1;
	    Chain tableSet = tsTable[tsIndex][0];
	    Chain tsStatus = tsTable[tsIndex][1];
	    
	    int tsAction = showTableSetActionMenu(tableSet, tsStatus);
	    if ( tsAction > 0 )
	    {
		if ( tsAction == TS_EDIT )
		{
		    showTableSetEditMask(tableSet);
		}		
		else if ( tsAction == TS_FILE )
		{
		    showTableSetFileInfo(tableSet);
		}
		else if ( tsAction == TS_BACKUP )
		{
		    showTableSetBackupInfo(tableSet);
		}
		else if ( tsAction == TS_ARCHLOG )
		{
		    showTableSetListArchlog(tableSet);
		}		
		else if ( tsAction == TS_QUERYCACHE )
		{
		    showTableSetQueryCache(tableSet);
		}		
		else if ( tsAction == TS_EXPORT )
		{
		    showTableSetExport(tableSet);
		}
		else if ( tsAction == TS_IMPORT )
		{
		    showTableSetImport(tableSet);
		}
		else
		{			
		    performTableSetAction(tableSet, tsAction);
		    if ( tsAction == TS_REMOVE_ACTION )
			setSelectedRow(1);
		}
	    }
	}
	else if ( ret == INFO_ACTION)
	{
	    int tsIndex = getSelectedRow() - 1;
	    Chain tableSet = tsTable[tsIndex][0];
	    showTableSetInfo(tableSet);
	}
	else if ( ret == ADD_ACTION)
	{
	    showTableSetDefineMask();
	}
	else if ( ret == FILEINFO_ACTION)
	{
	    int tsIndex = getSelectedRow() - 1;

	    Chain tableSet = tsTable[tsIndex][0];
	    showTableSetFileInfo(tableSet);
	}
    }

    setStatusLine(Chain());
}

void CegoAdmScreen::showTableSetInfo(const Chain& tableSet)
{

    bool go = true;
    while ( go ) 
    {
	setStatusLine(Chain("i,<ret> - close info"), _refInterval / 1000 + 1);

	ListT< ListT<CegoFieldValue> > tsInfo;
	
	CegoAdminHandler::ResultType res;
	CegoTableObject oe;	
	
	res = _pAH->medGetDetailedTableSetInfo(tableSet);	
	
	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    res = _pAH->nextInfo();	    
	}
	
	_pAH->getDetailedTableSetInfo(oe, tsInfo);
	
	ListT<Chain> tsMonInfo1;
	ListT<Chain> tsMonInfo2;
	
	ListT<CegoFieldValue> *pFVL = tsInfo.First();
	int row=1;
	while ( pFVL )
	{
	    Chain key, value;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {		       
		key = pFV->valAsChain();
	    }
	    
	    pFV = pFVL->Next();
	    if ( pFV )
	    {
		value = pFV->valAsChain();
	    }

	    if ( row < 25 )
		tsMonInfo1.Insert(key + VALSEPTOKEN + value);
	    else
		tsMonInfo2.Insert(key + VALSEPTOKEN + value);
	    
	    pFVL = tsInfo.Next();
	    row++;
	}

	ListT< ListT<Chain> > tsMonInfoAll;
	tsMonInfoAll.Insert(tsMonInfo1);
	tsMonInfoAll.Insert(tsMonInfo2);

	int ret = showAttributeBox(20, 30, tsMonInfoAll, false, _refInterval);

	if ( ret == INFO_ACTION
	     || ret == SC_LEAVE )
	    go=false;
	
    }
    return;
}


////////////////////
// Tableset Basic //
////////////////////


void CegoAdmScreen::showTableSetDefineMask()
{    
    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
	
    ListT<Chain> attrList; 
    ListT<Chain> valList; 
    attrList.Insert( Chain("Name") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain("TabNetName"));
    attrList.Insert( Chain("Root Path") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXPATHLEN + ATTRSEPTOKEN + Chain("./RootPath"));
    attrList.Insert( Chain("Primary") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN);
    attrList.Insert( Chain("Secondary") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN);
    attrList.Insert( Chain("SystemSpace") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("100"));
    attrList.Insert( Chain("TempSpace") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("100"));
    attrList.Insert( Chain("AppSpace") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("500"));
    attrList.Insert( Chain("LogSize") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("1000000"));
    attrList.Insert( Chain("LogNum") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("3"));
    attrList.Insert( Chain("Sort Area Size") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("1000000"));
    
    int r = showFormBox("Tableset Definition", attrList, valList, 30);
    
    if ( r )
    {	
	Chain tableSet = valList[0];
	Chain tsRoot = valList[1];
	Chain primary = valList[2];
	Chain secondary = valList[3];
	int sysSize = valList[4].asInteger();
	int tmpSize = valList[5].asInteger();
	int appSize = valList[6].asInteger();
	int logSize = valList[7].asInteger();
	int logNum = valList[8].asInteger();
	unsigned long long sortAreaSize = valList[9].asUnsignedLongLong();

	CegoAdminHandler::ResultType res;
	res = _pAH->medDefineTableSet(tableSet,
				      tsRoot,
				      primary,
				      secondary,
				      sysSize,
				      tmpSize,
				      appSize,
				      logSize,
				      logNum, 
				      sortAreaSize);
	
	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}


void CegoAdmScreen::showTableSetEditMask(const Chain& tableSet)
{    

    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    ListT< ListT<CegoFieldValue> > tsInfo;
    
    CegoAdminHandler::ResultType res;
    CegoTableObject oe;	
    
    res = _pAH->medGetDetailedTableSetInfo(tableSet);	
    
    while ( res == CegoAdminHandler::ADM_INFO )
    {
	res = _pAH->nextInfo();	    
    }
    
    _pAH->getDetailedTableSetInfo(oe, tsInfo);
    
    ListT<Chain> tsMonInfo;

    Chain actTSRoot;
    int actSysSize;
    int actTmpSize;
    int actAppSize;
    int actSortAreaSize;
    int actLogSize;
    int actLogNum=0;

    ListT<CegoFieldValue> *pFVL = tsInfo.First();
    while ( pFVL )
    {
	Chain key, value;
	
	CegoFieldValue *pFV = pFVL->First();
	if ( pFV )
	{		       
	    key = pFV->valAsChain();
	}
	
	pFV = pFVL->Next();
	if ( pFV )
	{
	    value = pFV->valAsChain();
	}

	if ( key == Chain(INFO_ROOTPATH) )
	    actTSRoot=value;
	else if ( key == Chain(INFO_SYSTEMPAGETOTAL) )
	    actSysSize=value.asInteger();
	else if ( key == Chain(INFO_TEMPPAGETOTAL) )
	    actTmpSize=value.asInteger();
	else if ( key == Chain(INFO_APPPAGETOTAL) )
	    actAppSize=value.asInteger();
	else if ( key == Chain(INFO_SORTAREASIZE) )
	    actSortAreaSize=value.asInteger();
	else if ( key == Chain(INFO_LOGSIZE) )
	    actLogSize=value.asInteger();
	else if ( key == Chain(INFO_LOGFILE) )
	    actLogNum++;;

	pFVL = tsInfo.Next();
    }


    ListT<Chain> attrList; 
    ListT<Chain> valList; 
    attrList.Insert( Chain("Root Path") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXPATHLEN + ATTRSEPTOKEN + actTSRoot);
    attrList.Insert( Chain("SystemSpace") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(actSysSize));
    attrList.Insert( Chain("TempSpace") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(actTmpSize));
    attrList.Insert( Chain("AppSpace") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(actAppSize));
    attrList.Insert( Chain("LogSize") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(actLogSize));
    attrList.Insert( Chain("LogNum") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(actLogNum));
    attrList.Insert( Chain("Sort Area Size") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(actSortAreaSize));
    
    int r = showFormBox("Tableset Edit " + tableSet, attrList, valList, 30);
    
    if ( r )
    {	
	Chain tsRoot = valList[0];
	int sysSize = valList[1].asInteger();
	int tmpSize = valList[2].asInteger();
	int appSize = valList[3].asInteger();
	int logSize = valList[4].asInteger();
	int logNum = valList[5].asInteger();
	unsigned long long sortAreaSize = valList[6].asUnsignedLongLong();


	CegoAdminHandler::ResultType res;
	res = _pAH->reqSetTSRootPath(tableSet, tsRoot);	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->reqSetTSSysSize(tableSet, sysSize);
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->reqSetTSTmpSize(tableSet, tmpSize);
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->reqSetTSAppSize(tableSet, appSize);
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->reqSetTSLogSize(tableSet, logSize);
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}
	
	res = _pAH->reqSetTSLogNum(tableSet, logNum);
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->reqSetTSSortAreaSize(tableSet, sortAreaSize);
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	    
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}
    }
}

///////////////////
// Tableset File //
///////////////////

void CegoAdmScreen::showTableSetFileInfo(const Chain& tableSet)
{

    int ret = SC_TIMEOUT;
    while ( ret != SC_LEAVE )
    {

	setStatusLine(Chain("a - add new datafile"));
	
	CegoAdminHandler::ResultType res;	
	res = _pAH->medGetDetailedTableSetInfo(tableSet);	
	
	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    res = _pAH->nextInfo();	    
	}
	
	ListT<ListT<CegoFieldValue> > fileInfo;
	CegoTableObject oe;
	Chain msg;
	Chain format;
	_pAH->getMsg(msg);
	_pAH->getDataFileInfo(oe, fileInfo, format);
	
	int maxPathLen=0;
	ListT<CegoFieldValue>* pFVL = fileInfo.First();	
	while ( pFVL )
	{	    
	    CegoFieldValue *pFV = pFVL->First();
	    if (pFV )
	    {		
		Chain fileName=(char*)pFV->valAsChain();
		if ( fileName.length() > maxPathLen )
		    maxPathLen = fileName.length();		
	    }
	    pFVL = fileInfo.Next();	
	}
	       
	// int width = maxPathLen+30;
	// int height = fileInfo.Size() +3;
		
	ListT<Chain> fiSchema;
	fiSchema.Insert(Chain("Name") + VALSEPTOKEN + Chain("50"));
	fiSchema.Insert(Chain("Type") + VALSEPTOKEN + Chain("10"));
	fiSchema.Insert(Chain("Size") + VALSEPTOKEN + Chain("10"));
	fiSchema.Insert(Chain("Used") + VALSEPTOKEN + Chain("10"));
	
	ListT< ListT<Chain> > fiTable;
	
	pFVL = fileInfo.First();
    	
	while ( pFVL )
	{
	    ListT<Chain> fiRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while (pFV )
	    {
		fiRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    fiTable.Insert(fiRow);
	    
	    pFVL = fileInfo.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

	ret = showTableBox(tableSet + Chain(" files"), fiSchema, fiTable,  colorMap, 0,
			   false, 0, 0, 0);

	if ( ret == ADD_ACTION )
	{
	    showTableSetAddFile(tableSet);
	}
    }
}

void CegoAdmScreen::showTableSetAddFile(const Chain&  tableSet)
{
    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));

    ListT<Chain> attrList; 
    ListT<Chain> valList;

    Chain typeMenuString = Chain("app") + VALSEPTOKEN + Chain("1")
	+ LISTSEPTOKEN + Chain("app") + VALSEPTOKEN + Chain("1")
	+ LISTSEPTOKEN + Chain("temp") + VALSEPTOKEN + Chain("2")
	+ LISTSEPTOKEN + Chain("sys") + VALSEPTOKEN + Chain("3");
    
    attrList.Insert( Chain("Type") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + typeMenuString);
    attrList.Insert( Chain("Path") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXPATHLEN + ATTRSEPTOKEN + Chain("./FilePath"));
    attrList.Insert( Chain("NumPages") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain("1000"));

    int r = showFormBox(Chain("Add datafile to ") + tableSet, attrList, valList, 30);
        
    if ( r )
    {	
	Chain fileType = valList[0];
	Chain dataFile = valList[1];
	Chain numPages = valList[2];
	
	bool typeCheck = true;
	if ( fileType == Chain("1" ) )
	{
	    fileType = Chain(XML_APPFILE_VALUE);
	}
	else if ( fileType == Chain("2" ) )
	{
	    fileType = Chain(XML_TEMPFILE_VALUE);
	}
	else if ( fileType == Chain("3" ) )
	{
	    fileType = Chain(XML_SYSFILE_VALUE);
	}
	else
	{
	    typeCheck=false;
	    showInfoBox(ERROR_TITLE, "Invalid file type, must be either app, temp or sys", MSGBOX_WIDTH);
	}
	
	if ( typeCheck )
	{
	    CegoAdminHandler::ResultType res;
	
	    res = _pAH->medAddDataFile(tableSet, fileType, dataFile, numPages.asInteger());
	    
	    Chain msg;
	    _pAH->getMsg(msg);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    }
	    else
	    {
		// showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	    }
	}
    }
}

/////////////////////////
// Tableset Archivelog //
/////////////////////////

void CegoAdmScreen::showTableSetListArchlog(const Chain& tableSet)
{
    int ret = SC_TIMEOUT;
    while ( ret != SC_LEAVE )
    {

	setStatusLine(Chain("a - add destination, c - configure, d - delete destination"));
	    
	CegoAdminHandler::ResultType res;	
	res = _pAH->medGetDetailedTableSetInfo(tableSet);	

	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    res = _pAH->nextInfo();	    
	}

	CegoTableObject oe;

	ListT<ListT<CegoFieldValue> > tsInfo;	
	_pAH->getDetailedTableSetInfo(oe, tsInfo);

	Chain archMode;
	ListT<CegoFieldValue> *pFVL = tsInfo.First();
	while ( pFVL )
	{
	    Chain key, value;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {		       
		key = pFV->valAsChain();
	    }
	    
	    pFV = pFVL->Next();
	    if ( pFV )
	    {
		value = pFV->valAsChain();
	    }
	    if ( key == Chain(INFO_ARCHMODE) )
		archMode = value;
	    
	    pFVL = tsInfo.Next();
	}
    

	ListT<ListT<CegoFieldValue> > archLogInfo;	
	_pAH->getArchLogInfo(oe, archLogInfo);

	
	int maxPathLen=0;
	pFVL = archLogInfo.First();	
	while ( pFVL )
	{	    
	    CegoFieldValue *pFV = pFVL->First();
	    pFV = pFVL->Next();
	    if (pFV )
	    {		
		Chain archLogPath=(char*)pFV->valAsChain();
		
		if ( archLogPath.length() > maxPathLen )
		    maxPathLen = archLogPath.length();		
	    }
	    pFVL = archLogInfo.Next();	
	}
	
	
	// int width = maxPathLen+30;
	// int height = archLogInfo.Size() +3;
	
	
	ListT<Chain> alSchema;
	alSchema.Insert(Chain("ArchId") + VALSEPTOKEN + Chain("20"));
	alSchema.Insert(Chain("Path") + VALSEPTOKEN + Chain("50"));
	
	ListT< ListT<Chain> > alTable;
	
	pFVL = archLogInfo.First();
    	
	while ( pFVL )
	{
	    ListT<Chain> alRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while (pFV )
	    {
		alRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    alTable.Insert(alRow);
	    
	    pFVL = archLogInfo.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

	ret = showTableBox(tableSet + Chain(" archive destinations - Status ") + archMode, alSchema, alTable,  colorMap);

	if ( ret == ADD_ACTION )
	{	    
	    showTableSetAddArchlog(tableSet);
	}
	else if ( ret == DELETE_ACTION )
	{
	    int archIndex = getSelectedRow() - 1;
	    Chain archId = alTable[archIndex][0];
	    showTableSetRemoveArchlog(tableSet, archId);
	}
	else if ( ret == CONF_ACTION )
	{
	    showTableSetToggleArchlog(tableSet, archMode);
	}
    }
}

void CegoAdmScreen::showTableSetAddArchlog(const Chain& tableSet)
{
    ListT<Chain> attrList; 
    ListT<Chain> valList;

    attrList.Insert( Chain("ArchId") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain("ALID"));
    attrList.Insert( Chain("Path") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXPATHLEN + ATTRSEPTOKEN + Chain("./FilePath"));

    int r = showFormBox(Chain("Add archive destination to ") + tableSet, attrList, valList,30);
        
    if ( r )
    {	
	Chain archId = valList[0];
	Chain archPath = valList[1];

	CegoAdminHandler::ResultType res;    
	res = _pAH->medAddArchLog(tableSet, archId, archPath);
	    
	Chain msg;
	_pAH->getMsg(msg);
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}      
    }
}

void CegoAdmScreen::showTableSetRemoveArchlog(const Chain& tableSet, const Chain& archId)
{
    
    int r = showConfirmBox(Chain("Remove archive distination ") + archId + Chain(" from ") + tableSet + Chain(" ?"), Chain());
    
    if ( r )
    {
	
	CegoAdminHandler::ResultType res;   
	res = _pAH->medRemoveArchLog(tableSet, archId);
		
	Chain msg;
	_pAH->getMsg(msg);
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

void CegoAdmScreen::showTableSetToggleArchlog(const Chain& tableSet, const Chain& archMode)
{
    ListT<Chain> attrList; 
    ListT<Chain> valList;

    Chain valString;
    if ( archMode == Chain(XML_ON_VALUE) )
	valString = Chain("Enabled") + VALSEPTOKEN + Chain("1");
    else
	valString = Chain("Disabled") + VALSEPTOKEN + Chain("2");
    
    Chain modeMenuString = valString + LISTSEPTOKEN
	+ Chain("Enabled") + VALSEPTOKEN + Chain("1") + LISTSEPTOKEN
	+ Chain("Disabled") + VALSEPTOKEN + Chain("2");
        
    attrList.Insert( Chain("Status") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + modeMenuString);
    
    int r = showFormBox(Chain("Toggle archlog for ") + tableSet, attrList, valList);
        
    if ( r )
    {	
	Chain mode = valList[0];

	CegoAdminHandler::ResultType res;   
	if ( mode == Chain("1"))
	{
	    res = _pAH->medEnableArchLog(tableSet);
	}
	else
	{
	    res = _pAH->medDisableArchLog(tableSet);
	}
	Chain msg;
	_pAH->getMsg(msg);
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}


//////////////////////////////////
// Tableset Backup and Recovery //
//////////////////////////////////

void CegoAdmScreen::showTableSetBackupInfo(const Chain& tableSet)
{

    int ret = SC_TIMEOUT;
    while ( ret != SC_LEAVE )
    {
	CegoAdminHandler::ResultType res;
	Chain pit;
	res = _pAH->medListBackup(tableSet, pit);

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	CegoTableObject oe;
	ListT<ListT<CegoFieldValue> > buInfo;

	_pAH->getBackupInfo(oe, buInfo);

	
	ListT<Chain> buSchema;
	buSchema.Insert(Chain("BackupId") + VALSEPTOKEN + Chain("20"));
	buSchema.Insert(Chain("Timestamp") + VALSEPTOKEN + Chain("20"));
		
	ListT<ListT<Chain> > buTable;
		   
	ListT<CegoFieldValue> *pFVL = buInfo.First();
	while ( pFVL )
	{
	    ListT<Chain> buRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while ( pFV )
	    {
		buRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    buTable.Insert(buRow);
	    pFVL = buInfo.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	
	ret = showTableBox(Chain("Backup List"), buSchema, buTable, colorMap, 0, true, 1, 1, 0);
	if ( ret == SC_SELECT )
	{
	    if ( buTable.Size() > 0 )
	    {
		int buIndex = getSelectedRow() - 1;       	
		Chain buId = buTable[buIndex][0];
		showTableSetBackupRecovery(tableSet, buId);
	    }
	}
	else if ( ret == ADD_ACTION )
	{
	    showTableSetBackupAdd(tableSet);
	}
    }
}

void CegoAdmScreen::showTableSetBackupAdd(const Chain& tableSet)
{

    int r = showConfirmBox(Chain("Adding online backup for tableset ") + tableSet + Chain(" ?"), Chain("This requires a configured backup manager"));
    
    if ( r )
    {

	CegoAdminHandler::ResultType res;
	Chain msg;

	Chain buMsg("Backup via CegoAdmScreen");
	
	res = _pAH->medBeginBackup(tableSet, buMsg);	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    _pAH->getMsg(msg);
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->medExecuteBackup(tableSet, buMsg);

	Chain errLog;
	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    _pAH->getMsg(msg);
	    errLog += msg + Chain("\n");
	    setStatusLine(msg);
	    res = _pAH->nextInfo();	    
	}
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);

	    errLog += msg;
	    
	    // try to end backup 
	    res = _pAH->medEndBackup(tableSet, buMsg, false);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		_pAH->getMsg(msg);
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
		return;
	    }

	    showInfoBox(ERROR_TITLE, errLog, MSGBOX_WIDTH);
	    return;
	}

	res = _pAH->medEndBackup(tableSet, buMsg, false);

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    _pAH->getMsg(msg);
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}
	else
	{
	    msg = Chain("Backup finished succesful");
	    setStatusLine(msg);
	}
    }
}

void CegoAdmScreen::showTableSetBackupRecovery(const Chain& tableSet, const Chain& buId)
{

    ListT<Chain> attrList; 
    ListT<Chain> valList;

    attrList.Insert( Chain("Point In Time") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain());
    
           
    int r = showFormBox(Chain("Recover tableset ") + tableSet + Chain(" from backup ") + buId + Chain(" ?"), attrList, valList);
    
    if ( r )
    {

	Chain pit = valList[0];
	
	if ( pit != Chain() )
	{
	    CegoAdminHandler::ResultType res;
	    res = _pAH->medListBackup(tableSet, pit);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		Chain msg;
		_pAH->getMsg(msg);	
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
		return;
	    }
	    
	    CegoTableObject oe;
	    ListT<ListT<CegoFieldValue> > buInfo;
	    
	    _pAH->getBackupInfo(oe, buInfo);

	    bool buFound = false;
	    ListT<CegoFieldValue> *pFVL = buInfo.First();
	    while ( pFVL && buFound == false)
	    {
		ListT<Chain> buRow;
		
		CegoFieldValue *pFV = pFVL->First();
		if ( pFV )
		{
		    Chain availId = (pFV->valAsChain());

		    if ( buId == availId )
			buFound = true;		    
		    pFV = pFVL->Next();
		}
		
		pFVL = buInfo.Next();
	    }

	    if ( buFound == false )
	    {
		showInfoBox(ERROR_TITLE, Chain("Cannot reach Pit=") + pit + Chain(" with backup ") + buId,
			    MSGBOX_WIDTH);
		return;		
	    }
	}

	
	CegoAdminHandler::ResultType res;
	res = _pAH->medRestore(tableSet, buId);
	    
	Chain msg;
	
	while ( res == CegoAdminHandler::ADM_INFO )
	{	
	    _pAH->getMsg(msg);
	    setStatusLine(msg);
	    res = _pAH->nextInfo();
	}
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    _pAH->getMsg(msg);
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}
	
	res = _pAH->medRecover(tableSet, pit);
	
	_pAH->getMsg(msg);
	
	while ( res == CegoAdminHandler::ADM_INFO )
	{	
	    _pAH->getMsg(msg);
	    setStatusLine(msg);
	    res = _pAH->nextInfo();
	}
    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    _pAH->getMsg(msg);
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    return;
	}
    }
}

////////////////////////////////
// Tableset Export and Import //
////////////////////////////////

void CegoAdmScreen::showTableSetExport(const Chain& tableSet)
{
    ListT<Chain> attrList; 
    ListT<Chain> valList;
    

    Chain modeMenuString = Chain("xml") + VALSEPTOKEN + Chain(XML_XML_VALUE) + LISTSEPTOKEN
	+ Chain("xml") + VALSEPTOKEN + Chain(XML_XML_VALUE) + LISTSEPTOKEN
	+ Chain("bin") + VALSEPTOKEN + Chain(XML_BIN_VALUE) + LISTSEPTOKEN
	+ Chain("plain") + VALSEPTOKEN + Chain(XML_PLAIN_VALUE);
    
    
    attrList.Insert( Chain("Mode") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + modeMenuString);
    attrList.Insert( Chain("File") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain());
    
    int r = showFormBox(Chain("Export tableset ") + tableSet, attrList, valList, 30);
        
    if ( r )
    {	
	Chain mode = valList[0];
	Chain exportFile = valList[1];

	setStatusLine(Chain("Export ") + tableSet + Chain(" to ") + mode + Chain(" ") + exportFile);
	
	CegoAdminHandler::ResultType res;
	bool isStructure = false;
	res = _pAH->reqExportTableSet(tableSet, isStructure, exportFile, mode);
	
	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	    
	    setStatusLine(tableSet + Chain(" Export : ") + msg);
	    res = _pAH->nextInfo();	    
	}

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	    
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

void CegoAdmScreen::showTableSetImport(const Chain& tableSet)
{
    ListT<Chain> attrList; 
    ListT<Chain> valList;
    
    Chain modeMenuString = Chain("xml") + VALSEPTOKEN + Chain(XML_XML_VALUE) + LISTSEPTOKEN
	+ Chain("xml") + VALSEPTOKEN + Chain(XML_XML_VALUE) + LISTSEPTOKEN
	+ Chain("bin") + VALSEPTOKEN + Chain(XML_BIN_VALUE) + LISTSEPTOKEN
	+ Chain("plain") + VALSEPTOKEN + Chain(XML_PLAIN_VALUE);
    
    attrList.Insert( Chain("Mode") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + modeMenuString);
    attrList.Insert( Chain("File") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain());
    
    int r = showFormBox(Chain("Import tableset ") + tableSet, attrList, valList, 30);
        
    if ( r )
    {	
	Chain mode = valList[0];
	Chain importFile = valList[1];
	
	CegoAdminHandler::ResultType res;
	bool isStructure = false;
	res = _pAH->reqImportTableSet(tableSet, isStructure, importFile, mode);
	
	while ( res == CegoAdminHandler::ADM_INFO )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	    
	    setStatusLine(tableSet + Chain(" Import : ") + msg);
	    res = _pAH->nextInfo();	    
	}

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    Chain msg;
	    _pAH->getMsg(msg);	    
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

///////////////////////////////
// Tableset Query Cache Menu //
///////////////////////////////

void CegoAdmScreen::showTableSetQueryCache(const Chain& tableSet)
{
    ListT< ListT<CegoFieldValue> > tsInfo;
    CegoAdminHandler::ResultType res;
    CegoTableObject oe;	
    
    res = _pAH->medGetDetailedTableSetInfo(tableSet);	
    
    while ( res == CegoAdminHandler::ADM_INFO )
    {
	res = _pAH->nextInfo();	    
    }

    _pAH->getDetailedTableSetInfo(oe, tsInfo);
    
    int maxEntry=0;
    int maxSize=0;
    int hashRange=0;
    Chain status;
    
    ListT<CegoFieldValue> *pFVL = tsInfo.First();
    while ( pFVL )
    {

	Chain key, value;
	
	CegoFieldValue *pFV = pFVL->First();
	if ( pFV )
	{		       
	    key = pFV->valAsChain();
	}
	
	pFV = pFVL->Next();
	if ( pFV )
	{
	    value = pFV->valAsChain();
	}
	if ( key == Chain(INFO_QUERYCACHE) )
	    status = value;
	if ( key == Chain(INFO_QUERYCACHEMAXENTRY) )
	    maxEntry=value.asInteger();
	else if ( key == Chain(INFO_QUERYCACHEMAXSIZE) )
	    maxSize=value.asInteger();
	else if ( key == Chain(INFO_QUERYCACHEHASHRANGE) )
	    hashRange=value.asInteger();
	
	pFVL = tsInfo.Next();
    }
        
    ListT<Chain> attrList;
    ListT<Chain> valList;

    Chain statusMenuString;

    statusMenuString += status + VALSEPTOKEN + status;
    statusMenuString += LISTSEPTOKEN;
    statusMenuString += XML_OFF_VALUE + VALSEPTOKEN + XML_OFF_VALUE;
    statusMenuString += LISTSEPTOKEN;
    statusMenuString += XML_ON_VALUE + VALSEPTOKEN + XML_ON_VALUE;

    attrList.Insert(Chain("Status") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + statusMenuString); 
    
    attrList.Insert(Chain("Max Entry") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(maxEntry));
    attrList.Insert(Chain("Max Size") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(maxSize));
    attrList.Insert(Chain("Hash Range") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(hashRange));


    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    int r = showFormBox("Query Cache Parameter", attrList, valList);
    
    if ( r )
    {
	status = valList[0];
	maxEntry = valList[1].asInteger();
	maxSize = valList[2].asInteger();
	hashRange = valList[3].asInteger();
	
	res = _pAH->reqSetQueryCacheParam(tableSet, maxEntry, maxSize, hashRange);
	
	Chain msg;

	_pAH->getMsg(msg);
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	
	if ( status == Chain(XML_ON_VALUE) )
	    res = _pAH->reqEnableQueryCache(tableSet);
	else
	    res = _pAH->reqDisableQueryCache(tableSet);

	_pAH->getMsg(msg);
	
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}	
    }
}

void CegoAdmScreen::performTableSetAction(const Chain& tableSet, int action)
{
    if ( action == TS_START_ACTION )
    {
	bool doCleanup=false;
	bool doForceload=false;
	bool cpDump=false;
	bool noInit=false;
	
	CegoAdminHandler::ResultType res;
	res = _pAH->medStartTableSet(tableSet, doCleanup, doForceload, cpDump, noInit);
		
	Chain msg;
	_pAH->getMsg(msg);

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}	
    }
    else if ( action == TS_STOP_ACTION )
    {
	CegoAdminHandler::ResultType res;
	res = _pAH->medStopTableSet(tableSet);
		
	Chain msg;
	_pAH->getMsg(msg);

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}
    }
    else if ( action == TS_CREATE_ACTION )
    {
	CegoAdminHandler::ResultType res;
	res = _pAH->medCreateTableSet(tableSet);
		
	Chain msg;
	_pAH->getMsg(msg);

	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}
    }
    else if ( action == TS_DROP_ACTION )
    {

	int r = showConfirmBox(Chain("Drop tableset ") + tableSet + Chain(" ?"), Chain());
	
	if ( r )
	{	
	    
	    CegoAdminHandler::ResultType res;
	    res = _pAH->medDropTableSet(tableSet);
	    
	    Chain msg;
	    _pAH->getMsg(msg);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    }
	}
    }
    else if ( action == TS_REMOVE_ACTION )
    {

	int r = showConfirmBox(Chain("Remove tablset ") + tableSet + Chain(" ?"), Chain());		    
	if ( r )
	{	
	    
	    CegoAdminHandler::ResultType res;
	    res = _pAH->medRemoveTableSet(tableSet);
	    
	    Chain msg;
	    _pAH->getMsg(msg);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    }
	}
    }
}

int CegoAdmScreen::showTableSetActionMenu(const Chain& tableSet, const Chain& tsStatus)
{
    setStatusLine(Chain("<ret> - start selected action"));
    
    ListT<Chain> tsActionList;	       	     	           
    if ( tsStatus == Chain(TS_DEFINED_STATUS) )
    {
	tsActionList.Insert(Chain("Edit") + VALSEPTOKEN + Chain(TS_EDIT));
	tsActionList.Insert(Chain("Create") + VALSEPTOKEN + Chain(TS_CREATE_ACTION));
	tsActionList.Insert(Chain("Remove") + VALSEPTOKEN + Chain(TS_REMOVE_ACTION));	
    }
    else if ( tsStatus == Chain(TS_OFFLINE_STATUS) )
    {	
	tsActionList.Insert(Chain("Start") + VALSEPTOKEN + Chain(TS_START_ACTION));
	tsActionList.Insert(Chain("Drop") + VALSEPTOKEN + Chain(TS_DROP_ACTION));
	tsActionList.Insert(Chain("Query Cache") + VALSEPTOKEN + Chain(TS_QUERYCACHE));
    }
    else if ( tsStatus == Chain(TS_ONLINE_STATUS) )
    {
	tsActionList.Insert(Chain("Stop tableset") + VALSEPTOKEN + Chain(TS_STOP_ACTION));
	tsActionList.Insert(Chain("Export") + VALSEPTOKEN + Chain(TS_EXPORT));
	tsActionList.Insert(Chain("Import") + VALSEPTOKEN + Chain(TS_IMPORT));
    }

    tsActionList.Insert(Chain("Datafile") + VALSEPTOKEN + Chain(TS_FILE));
    tsActionList.Insert(Chain("Archlog") + VALSEPTOKEN + Chain(TS_ARCHLOG));
    tsActionList.Insert(Chain("Backup") + VALSEPTOKEN + Chain(TS_BACKUP));
	

    Chain selectKey, selectValue;
    int ret =  showSelectBox(tableSet + Chain(" Tableset Action"), tsActionList, selectKey, selectValue);
    if ( ret )
    {
	return selectValue.asInteger();
    }
    return 0;
}

void CegoAdmScreen::showRefreshMenu()
{
    setStatusLine(Chain("<ret> - set refesh interval"));
    
    ListT<Chain> refreshList;
    refreshList.Insert(Chain("1 sec") + VALSEPTOKEN + Chain("1000"));
    refreshList.Insert(Chain("3 sec") + VALSEPTOKEN + Chain("3000"));
    refreshList.Insert(Chain("10 sec") + VALSEPTOKEN + Chain("10000"));
    refreshList.Insert(Chain("30 sec") + VALSEPTOKEN + Chain("30000"));
    
    Chain selectKey, selectValue;
    int ret = showSelectBox("Select refresh interval", refreshList, selectKey, selectValue);
    if ( ret )
    {
	_refInterval = selectValue.asInteger();
    }
}


void CegoAdmScreen::showTableSetHelp()
{
    /*
    Chain title("Tableset Help Menu");

    int width = 70;
    int height = 10;
 
    int highlight=1;
    int choice = 0;
    int c;
    
    int startx = (_mainx - width) / 2;
    int starty = (_mainy - height) / 2;
    
    WINDOW* helpwin = newwin(height, width, starty, startx);

    noecho();

    keypad(helpwin, TRUE);

    refresh();
    
    int x, y, i;	
    wattron(helpwin, A_BOLD);
    mvwprintw(helpwin, 1, 1, "%s", (char*)title);
    wattroff(helpwin, A_BOLD);


    int row=3;
    int sepCol=18;
    int descCol=20;
    
    box(helpwin, 0, 0);
    
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Down");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select next tableset");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Up");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select previous tableset");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"i");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Get detailed information about selected tableset");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"n");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Define new tableset");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"RET");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Open tableset submenu");

    bool showIt=true;

    while( showIt )
    {	
	c = wgetch(helpwin);
	switch(c)
	{	
	case KEY_RETURN:
	{
	    showIt = false;
	    choice = highlight;
	    break;
	}
	case KEY_ESC:
	{
	    showIt = false;
	    break;
	}
	}
    }
    delwin(helpwin);

    */
    return;
	
}

///////////////
// User Menu //
///////////////


void CegoAdmScreen::showUser()
{
    int selectedRow=1;
    int visibleRow=1;
    Chain newUser;
    int ret = SC_TIMEOUT;
    while ( ret != SC_LEAVE )
    {

	setStatusLine(Chain("a - add user, <ret> - open user menu"));

	CegoAdminHandler::ResultType res;	
	res = _pAH->reqShowUser();

	CegoTableObject oe;
	ListT<ListT<CegoFieldValue> > userList;
	
	_pAH->getUserInfo(oe, userList);
	
	ListT<Chain> userSchema;
	userSchema.Insert(Chain("Name") + VALSEPTOKEN + Chain("20"));
	userSchema.Insert(Chain("Role") + VALSEPTOKEN + Chain("20"));
	userSchema.Insert(Chain("Trace") + VALSEPTOKEN + Chain("10"));
	userSchema.Insert(Chain("NumRequest") + VALSEPTOKEN + Chain("15"));
	userSchema.Insert(Chain("NumQuery") + VALSEPTOKEN + Chain("15"));
		
	ListT< ListT<Chain> > userTable;

	int row=1;
	ListT<CegoFieldValue> *pFVL = userList.First();
	while ( pFVL )
	{
	    ListT<Chain> userRow;
	    
	    CegoFieldValue *pFV = pFVL->First();

	    if ( pFV )
		if ( newUser == pFV->valAsChain() )
		    selectedRow=row;
	    
	    while ( pFV )
	    {
		userRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    userTable.Insert(userRow);
	    pFVL = userList.Next();
	    row++;
	}

	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

	ret = showTableBox(Chain("Users"), userSchema, userTable, colorMap, 0, true, selectedRow, visibleRow);
	selectedRow=getSelectedRow();
	visibleRow=getVisibleRow();

	if ( ret == SC_SELECT)
	{
	    int userIndex = getSelectedRow() - 1;
	    Chain usName = userTable[userIndex][0];
	    Chain roleListString = userTable[userIndex][1];
	    Chain usTrace = userTable[userIndex][2];
	    int usAction = showUserActionMenu();
	    if ( usAction > 0 )
	    {
		if ( usAction == USER_ASSIGNROLE_ACTION )
		{
		    ListT<Chain> roleList = roleString2List(roleListString);
		    showRoleAssignMask(usName, roleList);
		}
		else if ( usAction == USER_REMOVEROLE_ACTION )
		{
		    ListT<Chain> roleList = roleString2List(roleListString);
		    showRoleRemoveMask(usName, roleList);
		}
		else if ( usAction == USER_CHANGEPWD_ACTION )
		{
		    showUserChangePwd(usName);
		}
		else if ( usAction == USER_TOGGLETRACE_ACTION )
		{		    
		    CegoAdminHandler::ResultType res;

		    if ( usTrace == Chain("ON"))
		    {
			res = _pAH->reqUserTrace(usName, false);
		    }
		    else
		    {
			res = _pAH->reqUserTrace(usName, true);
		    }
		    
		    Chain msg;
		    _pAH->getMsg(msg);
		    
		    if ( res == CegoAdminHandler::ADM_ERROR )
		    {
			showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
		    }
		    else
		    {
			// showInfoBox(INFO_TITLE, Chain("User trace enabled"), MSGBOX_WIDTH);
		    }			   
		}
		else if ( usAction == USER_DELETE_ACTION )
		{
		    int r = showConfirmBox(Chain("Delete user ") + usName + Chain(" ?"), Chain());
		    if ( r )
		    {	
			
			CegoAdminHandler::ResultType res;
			
			res = _pAH->reqRemoveUser(usName);
			
			Chain msg;
			_pAH->getMsg(msg);
			if ( res == CegoAdminHandler::ADM_ERROR )
			{
			    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
			}			
		    }
		}
	    }		
	}
	else if ( ret == ADD_ACTION)
	{
	    newUser = showUserAddMask();
	}
    }
}

Chain CegoAdmScreen::showUserAddMask()
{

    Chain userName;
    
    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    ListT<Chain> attrList;
    ListT<Chain> valList;
    
    attrList.Insert(Chain("Name") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain());
    attrList.Insert(Chain("Passwd") + ATTRSEPTOKEN + INPUT_TYPE_PWD + ATTRSEPTOKEN + MAXPASSWDLEN + ATTRSEPTOKEN + Chain());
         
    int r = showFormBox("User Definition", attrList, valList);
    
    if ( r )
    {
	
	userName = valList[0];
	Chain passwd = valList[1];

	CegoAdminHandler::ResultType res;
    
	res = _pAH->reqAddUser(userName, passwd);
	
	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    userName=Chain();
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
    return userName;
}

int CegoAdmScreen::showUserActionMenu()
{
    setStatusLine(Chain("<ret> - start user action"));
    
    ListT<Chain> userActionList;	       	     	           
    
    userActionList.Insert(Chain("Toggle trace") + VALSEPTOKEN + Chain(USER_TOGGLETRACE_ACTION));
    userActionList.Insert(Chain("Assign role") + VALSEPTOKEN + Chain(USER_ASSIGNROLE_ACTION));
    userActionList.Insert(Chain("Remove role") + VALSEPTOKEN + Chain(USER_REMOVEROLE_ACTION));
    userActionList.Insert(Chain("Change Password") + VALSEPTOKEN + Chain(USER_CHANGEPWD_ACTION));       
    userActionList.Insert(Chain("Delete") + VALSEPTOKEN + Chain(USER_DELETE_ACTION));

    Chain selectKey, selectValue;
    int ret = showSelectBox("User Action", userActionList, selectKey, selectValue);
    if ( ret )
    {
	return selectValue.asInteger();
    }
    return 0;
}


void CegoAdmScreen::showRoleAssignMask(const Chain& userName, const ListT<Chain>& userRoleList) 
{ 
    ListT<Chain> attrList; 
    ListT<Chain> valList; 

    CegoAdminHandler::ResultType res; 
    res = _pAH->reqListRole(); 

    CegoTableObject oe; 
    ListT<ListT<CegoFieldValue> > roleList; 
    _pAH->getRoleList(oe, roleList); 

    ListT<CegoFieldValue> adminRoleList;
    adminRoleList.Insert(CegoFieldValue(VARCHAR_TYPE, XML_ADMIN_ROLE));
    roleList.Insert(adminRoleList);

    int ridx=0;
    Chain roleMenuString;
    bool isFirst=true;
    ListT<CegoFieldValue> *pFVL = roleList.First(); 
    while ( pFVL ) 
    {
	bool roleAdded=false;
        CegoFieldValue *pFV = pFVL->First(); 
        if ( pFV ) 
        {
	    if ( userRoleList.Find(pFV->valAsChain()) == 0)
	    {
		roleAdded=true;
		
		roleMenuString += pFV->valAsChain() + VALSEPTOKEN + Chain(ridx);
		if ( isFirst ) // add first value as default value
		    roleMenuString += LISTSEPTOKEN + pFV->valAsChain() + VALSEPTOKEN + Chain(ridx);
		isFirst=false;
	    }
	}
	pFVL = roleList.Next(); 
        if ( pFVL && roleAdded) 
             roleMenuString += LISTSEPTOKEN; 
        ridx++; 
    } 

    attrList.Insert(Chain("Role") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + roleMenuString); 

    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
	
    int r = showFormBox(Chain("Assign role to ") + userName, attrList, valList); 
    
    if ( r ) 
    {
        int ridx = valList[0].asInteger(); 
        Chain roleName = roleList[ridx][0].valAsChain(); 
	
        CegoAdminHandler::ResultType res; 

        res = _pAH->reqAssignRole(userName, roleName);
	
	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

void CegoAdmScreen::showRoleRemoveMask(const Chain& userName, const ListT<Chain>& userRoleList) 
{
    ListT<Chain> attrList; 
    ListT<Chain> valList; 

    CegoAdminHandler::ResultType res; 
    res = _pAH->reqListRole(); 

    CegoTableObject oe; 
    ListT<ListT<CegoFieldValue> > roleList; 
    _pAH->getRoleList(oe, roleList); 

    ListT<CegoFieldValue> adminRoleList;
    adminRoleList.Insert(CegoFieldValue(VARCHAR_TYPE, XML_ADMIN_ROLE));
    roleList.Insert(adminRoleList);

    Chain roleMenuString; 
    int ridx=0; 

    ListT<CegoFieldValue> *pFVL = roleList.First(); 
    while ( pFVL ) 
    {
	bool roleAdded=false;
        CegoFieldValue *pFV = pFVL->First(); 
        if ( pFV ) 
        {
	    if ( userRoleList.Find(pFV->valAsChain()) != 0 )
	    {
		roleAdded=true;
		roleMenuString += pFV->valAsChain() + VALSEPTOKEN + Chain(ridx); 
	    }
	}
	pFVL = roleList.Next(); 
        if ( pFVL && roleAdded) 
             roleMenuString += LISTSEPTOKEN; 
        ridx++; 
    } 

    attrList.Insert(Chain("Role") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + roleMenuString); 

    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    int r = showFormBox(Chain("Remove role from ") + userName, attrList, valList);

    if ( r )
    {
	int ridx = valList[0].asInteger(); 
        Chain roleName = roleList[ridx][0].valAsChain();
	
	CegoAdminHandler::ResultType res;
	
	res = _pAH->reqRemoveRole(userName, roleName);

	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

void CegoAdmScreen::showUserChangePwd(const Chain& userName)
{    
    ListT<Chain> attrList;
    ListT<Chain> valList;

    attrList.Insert(Chain("Password") + ATTRSEPTOKEN + INPUT_TYPE_PWD + ATTRSEPTOKEN + MAXPASSWDLEN + ATTRSEPTOKEN + Chain());

    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    int r = showFormBox(Chain("Set password for ") + userName, attrList, valList);
    
    if ( r )
    {	
	Chain pwd = valList[0];
	
	CegoAdminHandler::ResultType res;
	
	res = _pAH->reqChangePwd(userName, pwd);

	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

////////////////
// Roles Menu //
////////////////

void CegoAdmScreen::showRole()
{
    int selectedRow=1;
    int visibleRow=1;
    int ret = SC_TIMEOUT;

    while ( ret != SC_LEAVE )
    {         

	setStatusLine(Chain("a - add role, <ret> - show role action"));
		
	CegoAdminHandler::ResultType res;	
	res = _pAH->reqListRole();
	
	CegoTableObject oe;
	ListT<ListT<CegoFieldValue> > roleList;	
	_pAH->getRoleList(oe, roleList);

	ListT<Chain> roleSchema;
	roleSchema.Insert(Chain("Name") + VALSEPTOKEN + Chain("50"));

	ListT< ListT<Chain> > roleTable;	
	ListT<CegoFieldValue> *pFVL = roleList.First();
	while ( pFVL )
	{
	    ListT<Chain> roleRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {
		roleRow.Insert(pFV->valAsChain());		
	    }
	    pFVL = roleList.Next();

	    roleTable.Insert(roleRow);
	}
	
	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));
	
	ret = showTableBox(Chain("Roles"), roleSchema, roleTable,  colorMap, 0, true, selectedRow, visibleRow);

	selectedRow=getSelectedRow();
	visibleRow=getVisibleRow();

	if ( ret == ADD_ACTION )
	{
	    showRoleAddMask();
	}
	else if ( ret == DELETE_ACTION )
	{
	    int roleIdx = getSelectedRow() - 1;	    
	    Chain roleName = roleTable[roleIdx][0];

	    int r = showConfirmBox(Chain("Delete role ") + roleName + Chain(" ?"), Chain());
	    
	    if ( r )
	    {	

		CegoAdminHandler::ResultType res;
		
		res = _pAH->reqDropRole(roleName);
		
		Chain msg;
		_pAH->getMsg(msg);
		if ( res == CegoAdminHandler::ADM_ERROR )
		{
		    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
		}
	    }
	}
	else if ( ret == SC_SELECT )
	{
	    int roleIdx = getSelectedRow() - 1;	    

	    Chain roleName = roleTable[roleIdx][0];	    

	    showRolePermListMask(roleName);
	    
	}
    }
}


void CegoAdmScreen::showRoleAddMask()
{
    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    ListT<Chain> attrList;
    ListT<Chain> valList;
    
    attrList.Insert(Chain("Name") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + Chain());
         
    int r = showFormBox("Role Definition", attrList, valList);
    
    if ( r )
    {	
	Chain roleName = valList[0];

	CegoAdminHandler::ResultType res;
    
	res = _pAH->reqCreateRole(roleName);
  	
	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
	else
	{
	    // showInfoBox(INFO_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}


void CegoAdmScreen::showRolePermListMask(const Chain& role)
{
    int selectedRow=1;
    int visibleRow=1;
    int ret = 1;
    
    while ( ret != SC_LEAVE )
    {
	setStatusLine(Chain("a - add/set permission, d - delete permission"));

	CegoAdminHandler::ResultType res;
	res = _pAH->reqShowRole(role);
		
	CegoTableObject oe;
	ListT<ListT<CegoFieldValue> > permInfo;
	
	_pAH->getRoleInfo(oe, permInfo);

	Chain msg;
	_pAH->getMsg(msg);
	
	ListT<Chain> permSchema;
	permSchema.Insert(Chain("PermId") + VALSEPTOKEN + Chain("20"));
	permSchema.Insert(Chain("TableSet") + VALSEPTOKEN + Chain("20"));
	permSchema.Insert(Chain("Filter") + VALSEPTOKEN + Chain("20"));
	permSchema.Insert(Chain("Permission") + VALSEPTOKEN + Chain("20"));
		
	ListT<ListT<Chain> > permTable;
		   
	ListT<CegoFieldValue> *pFVL = permInfo.First();
	while ( pFVL )
	{
	    ListT<Chain> permRow;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    while ( pFV )
	    {
		permRow.Insert(pFV->valAsChain());
		pFV = pFVL->Next();
	    }

	    permTable.Insert(permRow);
	    pFVL = permInfo.Next();
	}

	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));
	
	ret = showTableBox(role + Chain(" permissions"), permSchema, permTable, colorMap, 0, true, selectedRow, visibleRow);
	selectedRow=getSelectedRow();
	visibleRow=getVisibleRow();
      
	if ( ret == ADD_ACTION )
	{
	    showRolePermSetMask(role, Chain("P42"), Chain("TS1"), Chain("ALL"), Chain("ALL"));
	}
	else if ( ret == SC_SELECT )
	{
	    int permIdx = getSelectedRow() - 1;	    
	    Chain permId = permTable[permIdx][0];
	    Chain permTS = permTable[permIdx][1];
	    Chain permFilter = permTable[permIdx][2];
	    Chain permRight = permTable[permIdx][3];
	    	    
	    showRolePermSetMask(role, permId, permTS, permFilter, permRight);
	}
	else if ( ret == DELETE_ACTION )
	{
	    int permIdx = getSelectedRow() - 1;	    
	    Chain permId = permTable[permIdx][0];

	    int r = showConfirmBox(Chain("Delete Permission ") + permId + Chain(" ?"), Chain());
	    
	    if ( r )
	    {	
		
		CegoAdminHandler::ResultType res;
		res = _pAH->reqRemovePermission(role, permId);

		Chain msg;
		_pAH->getMsg(msg);
		
		if ( res == CegoAdminHandler::ADM_ERROR )
		{
		    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
		}
	    }
	}
    }
}

void CegoAdmScreen::showRolePermSetMask(const Chain& role,
					const Chain& permId,
					const Chain& permTS,
					const Chain& permFilter,
					const Chain& permRight)
{    
    setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
    
    ListT<Chain> attrList;
    ListT<Chain> valList;

    Chain rightMenuString = Chain(permRight) + VALSEPTOKEN + Chain(permRight)
	+ LISTSEPTOKEN + Chain("READ") + VALSEPTOKEN + Chain("READ")
	+ LISTSEPTOKEN + Chain("WRITE") + VALSEPTOKEN + Chain("WRITE")
	+ LISTSEPTOKEN + Chain("MODIFY") + VALSEPTOKEN + Chain("MODIFY")
	+ LISTSEPTOKEN + Chain("EXEC") + VALSEPTOKEN + Chain("EXEC")
	+ LISTSEPTOKEN + Chain("ALL") + VALSEPTOKEN + Chain("ALL");
        
    attrList.Insert(Chain("PermId") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + permId);
    attrList.Insert(Chain("TableSet") + ATTRSEPTOKEN + INPUT_TYPE_ID + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + permTS);
    attrList.Insert(Chain("Filter") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + permFilter);
    attrList.Insert( Chain("Right") + ATTRSEPTOKEN + INPUT_TYPE_MENU + ATTRSEPTOKEN + MAXNAMELEN + ATTRSEPTOKEN + rightMenuString);
         
    int r = showFormBox("Permission Definition", attrList, valList);
    
    if ( r )
    {	
	Chain permId = valList[0];
	Chain tableSet = valList[1];
	Chain filterPerm = valList[2];
	Chain right = valList[3];

	CegoAdminHandler::ResultType res;
	    
	res = _pAH->reqSetPermission(role, permId, tableSet, filterPerm, right);
	
	Chain msg;
	_pAH->getMsg(msg);
	    
	if ( res == CegoAdminHandler::ADM_ERROR )
	{
	    showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	}
    }
}

////////////////////////////
// Log and Backup Manager //
////////////////////////////

void CegoAdmScreen::showManager()
{
    int ret = SC_TIMEOUT;

    int selectedRow=1;
    
    ListT<Chain> mngSchema;
    mngSchema.Insert(Chain("Name") + VALSEPTOKEN + Chain("20"));
    
    ListT< ListT<Chain> > mngTable;	
    
    ListT<Chain> logRow;
    logRow.Insert("Log Manager");	
    mngTable.Insert(logRow);
    
    ListT<Chain> backupRow;
    backupRow.Insert("Backup Manager");	
    mngTable.Insert(backupRow);
        
    ListT<Screen::TableColor> colorMap;		   
    // colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));

    while ( ret != SC_LEAVE )
    {

	setStatusLine(Chain("<ret> - select manager to edit"));
	    
	ret = showTableBox(Chain("Manager"), mngSchema, mngTable, colorMap, 0, true, selectedRow ,1,2);
	selectedRow=getSelectedRow();
	
	if ( ret == SC_SELECT )
	{
	    int roleIdx = getSelectedRow() - 1;	    
	    
	    Chain mngName = mngTable[roleIdx][0];
	    
	    if ( mngName == Chain("Log Manager") )
	    {
		showLogManager();
	    }
	    else if ( mngName == Chain("Backup Manager") )
	    {
		showBackupManager();
	    }     
	}
    }
}


void CegoAdmScreen::showLogManager()
{

    CegoAdminHandler::ResultType res;
    res = _pAH->reqShowLogMng();

    if ( res == CegoAdminHandler::ADM_OK )
    {

	CegoTableObject oe;
	ListT<ListT<CegoFieldValue> > lmInfo;

	_pAH->getLogMngInfo(oe, lmInfo);

	int logTimeout=0;
	Chain logManager;
    
	ListT<CegoFieldValue> *pFVL = lmInfo.First();
	if ( pFVL )
	{
	    Chain key, value;
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {		       
		logManager = pFV->valAsChain();
	    }
	    
	    pFV = pFVL->Next();
	    if ( pFV )
	    {
		logTimeout = pFV->valAsChain().asInteger();
	    }	    
	}
    
	ListT<Screen::TableColor> colorMap;		   
	// colorMap.Insert(Screen::TableColor(Chain("Alpha"), SC_RED));
    
	
	ListT<Chain> attrList;
	ListT<Chain> valList;
	
    
	attrList.Insert(Chain("Log Manager") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXPATHLEN + ATTRSEPTOKEN + logManager);
	attrList.Insert(Chain("Timeout") + ATTRSEPTOKEN + INPUT_TYPE_NUMBER + ATTRSEPTOKEN + MAXSIZELEN + ATTRSEPTOKEN + Chain(logTimeout));


	setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
	
	int r = showFormBox("Log Manager Configuration", attrList, valList, 30);
    
	if ( r )
	{	   
	    logManager = valList[0];
	    logTimeout = valList[1].asInteger();

	    res = _pAH->reqSetLogMng(logManager, logTimeout);
	
	    Chain msg;

	    _pAH->getMsg(msg);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    }
	}
    }
}

void CegoAdmScreen::showBackupManager()
{
    CegoAdminHandler::ResultType res;
    res = _pAH->reqShowBackupMng();

    if ( res == CegoAdminHandler::ADM_OK )
    {

	CegoTableObject oe;
	ListT<ListT<CegoFieldValue> > lmInfo;

	_pAH->getBackupMngInfo(oe, lmInfo);

	Chain backupManager;
    
	ListT<CegoFieldValue> *pFVL = lmInfo.First();
	if ( pFVL )
	{
	    
	    CegoFieldValue *pFV = pFVL->First();
	    if ( pFV )
	    {		       
		backupManager = pFV->valAsChain();
	    }	    
	}
  	
	ListT<Chain> attrList;
	ListT<Chain> valList;
	    
	attrList.Insert(Chain("Backup Manager") + ATTRSEPTOKEN + INPUT_TYPE_STRING + ATTRSEPTOKEN + MAXPATHLEN + ATTRSEPTOKEN + backupManager);


	setStatusLine(Chain("<tab>,<cur-up>,<cur-down>,<cur-left>,<cur-right> to navigate"));
	
	int r = showFormBox("Backup Manager Configuration", attrList, valList, 30);
    
	if ( r )
	{	   
	    backupManager = valList[0];

	    res = _pAH->reqSetBackupMng(backupManager);
	
	    Chain msg;

	    _pAH->getMsg(msg);
	    
	    if ( res == CegoAdminHandler::ADM_ERROR )
	    {
		showInfoBox(ERROR_TITLE, msg, MSGBOX_WIDTH);
	    }
	}
    }
}

ListT<Chain> CegoAdmScreen::roleString2List(const Chain& roleString)
{
    Tokenizer t(roleString, Chain(","));

    Chain role;
    ListT<Chain> roleList;
    
    while ( t.nextToken(role))
	roleList.Insert(role);

    return roleList;		
}

/*
void CegoAdmScreen::showHelp()
{

    int width = HELPMASK_WIDTH;
    int height = HELPMASK_HEIGHT;
 
    int c;

    int startx = (_mainx - width) / 2;
    int starty = (_mainy - height) / 2;
    
    WINDOW *helpwin = newwin(height, width, starty, startx);

    noecho();

    keypad(helpwin, TRUE);

    wattron(helpwin, A_UNDERLINE);
    mvwprintw(helpwin, 1, 1, "Available commands");
    wattroff(helpwin, A_UNDERLINE);

    box(helpwin, 0, 0);

    int row=3;
    int sepCol=18;
    int descCol=20;

    wattron(helpwin, A_BOLD);
    mvwprintw(helpwin, row, 1, "%s", (char*)"Header Menu");
    wattroff(helpwin, A_BOLD);

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Left");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select left header menu");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Right");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select right header menu");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Return");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Open selected menu");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Esc");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Go back");

    row++;
    row++;
    wattron(helpwin, A_BOLD);
    mvwprintw(helpwin, row, 1, "%s", (char*)"Sub Menu");
    wattroff(helpwin, A_BOLD);

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Return");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Open option menu");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Esc");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Go back");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"n");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"New entry");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"i");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Get information about entry");

    row++;
    row++;
    wattron(helpwin, A_BOLD);
    mvwprintw(helpwin, row, 1, "%s", (char*)"Selection Menu");
    wattroff(helpwin, A_BOLD);

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Return");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select item action");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Esc");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Go back");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Down");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select next item");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Up");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select previous item");

    row++;
    row++;
    wattron(helpwin, A_BOLD);
    mvwprintw(helpwin, row, 1, "%s", (char*)"Input Mask");
    wattroff(helpwin, A_BOLD);

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Tab,Cursor Down");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select next field");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Up");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Select previous field");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Left");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Move input cursor left");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Cursor Right");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Move input cursor right");

    row++;
    mvwprintw(helpwin, row, 1, "%s", (char*)"Return");
    mvwprintw(helpwin, row, sepCol, "%s", (char*)"-");
    mvwprintw(helpwin, row, descCol, "%s", (char*)"Confirm or Abort");

    refresh();

    c = wgetch(helpwin);
   
    delwin(helpwin);

    clear();

}

*/
